import socket


def get_inner_ip():
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
        s.connect(('8.8.8.8', 80))
        return s.getsockname()[0]


def get_outer_ip():
    import requests
    return requests.get('http://ifconfig.me/ip', timeout=1).text.strip()


if __name__ == "__main__":
    print(get_inner_ip())
    print(get_outer_ip())



if __name__ == '__main__':
    import numpy as np
    import torch
    from torch import nn
    from torch.nn import init


    class ScaledDotProductAttention(nn.Module):
        '''
        Scaled dot-product attention
        '''

        def __init__(self, d_model, d_k, d_v, h, dropout=.1):
            '''
            :param d_model: Output dimensionality of the model
            :param d_k: Dimensionality of queries and keys
            :param d_v: Dimensionality of values
            :param h: Number of heads
            '''
            super(ScaledDotProductAttention, self).__init__()
            self.fc_q = nn.Linear(d_model, h * d_k)
            self.fc_k = nn.Linear(d_model, h * d_k)
            self.fc_v = nn.Linear(d_model, h * d_v)
            self.fc_o = nn.Linear(h * d_v, d_model)
            self.dropout = nn.Dropout(dropout)

            self.d_model = d_model
            self.d_k = d_k
            self.d_v = d_v
            self.h = h

            self.init_weights()

        def init_weights(self):
            for m in self.modules():
                if isinstance(m, nn.Conv2d):
                    init.kaiming_normal_(m.weight, mode='fan_out')
                    if m.bias is not None:
                        init.constant_(m.bias, 0)
                elif isinstance(m, nn.BatchNorm2d):
                    init.constant_(m.weight, 1)
                    init.constant_(m.bias, 0)
                elif isinstance(m, nn.Linear):
                    init.normal_(m.weight, std=0.001)
                    if m.bias is not None:
                        init.constant_(m.bias, 0)

        def forward(self, queries, keys, values, attention_mask=None, attention_weights=None):
            '''
            Computes
            :param queries: Queries (b_s, nq, d_model)
            :param keys: Keys (b_s, nk, d_model)
            :param values: Values (b_s, nk, d_model)
            :param attention_mask: Mask over attention values (b_s, h, nq, nk). True indicates masking.
            :param attention_weights: Multiplicative weights for attention values (b_s, h, nq, nk).
            :return:
            '''
            b_s, nq = queries.shape[:2]
            nk = keys.shape[1]
            print(queries.shape)

            q = self.fc_q(queries).view(b_s, nq, self.h, self.d_k).permute(0, 2, 1, 3)  # (b_s, h, nq, d_k)
            k = self.fc_k(keys).view(b_s, nk, self.h, self.d_k).permute(0, 2, 3, 1)  # (b_s, h, d_k, nk)
            v = self.fc_v(values).view(b_s, nk, self.h, self.d_v).permute(0, 2, 1, 3)  # (b_s, h, nk, d_v)

            att = torch.matmul(q, k) / np.sqrt(self.d_k)  # (b_s, h, nq, nk)
            if attention_weights is not None:
                att = att * attention_weights
            if attention_mask is not None:
                att = att.masked_fill(attention_mask, -np.inf)
            att = torch.softmax(att, -1)
            att = self.dropout(att)

            out = torch.matmul(att, v).permute(0, 2, 1, 3).contiguous().view(b_s, nq,
                                                                             self.h * self.d_v)  # (b_s, nq, h*d_v)
            out = self.fc_o(out)  # (b_s, nq, d_model)
            return out


    input=torch.randn(50,49,512) # batch_sez, seq_len(n_q), d_model
    sa = ScaledDotProductAttention(d_model=512, d_k=512, d_v=512, h=8)
    output=sa(input,input,input)
    print(output.shape)



import socket
import time
import _thread

socket.setdefaulttimeout(3)  # 设置默认超时时间


def socket_port(ip, port):
    """
    输入IP和端口号，扫描判断端口是否占用
    """
    try:
        if port >= 65535:
            print(u'端口扫描结束')
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = s.connect_ex((ip, port))
        if result == 0:
            lock.acquire()
            print(ip, u':', port, u'端口已占用')
            lock.release()
    except:
        print(u'端口扫描异常')


def ip_scan(ip):
    """
    输入IP，扫描IP的0-65534端口情况
    """
    try:
        print(u'开始扫描 %s' % ip)
        start_time = time.time()
        for i in range(0, 65534):
            _thread.start_new_thread(socket_port, (ip, int(i)))
        print(u'扫描端口完成，总共用时：%.2f' % (time.time() - start_time))
    # raw_input("Press Enter to Exit")
    except:
        print(u'扫描ip出错')


if __name__ == '__main__':
    url = input('Input the ip you want to scan: ')
    lock = _thread.allocate_lock()
    ip_scan(url)


from alive_progress import alive_bar
from rich.progress import Progress


class probar:
    bar_options = ["smooth", "classic", "classic2", "brackets", "blocks", "bubbles", "solid", "checks", "circles",
                   "squares", "halloween", "filling", "notes", "ruler", "ruler2", "fish", "scuba"]

    def __init__(self, iterable, total_steps=None, dual_line=True, title=None, bar='smooth', force_tty=None):
        """
        bar: probar.bar_options
        """
        self.iterable = iterable
        if total_steps is None:
            try:
                total_steps = len(iterable)
                if not total_steps: total_steps = None
            except:
                total_steps = None
        self.total_steps = total_steps
        self.dual_line = dual_line
        # self.ctrl_c = ctrl_c
        self.title = title
        self.bar = bar
        self.force_tty = force_tty

    def __iter__(self):
        with alive_bar(
                self.total_steps,
                bar=self.bar,
                dual_line=self.dual_line,
                title=self.title,
                force_tty=self.force_tty
        ) as pbar:
            for idx, item in enumerate(self.iterable):
                yield item
                pbar()


class probar2:
    def __init__(self, iterable, total_steps=None, title=None):
        """
        """
        self.iterable = iterable
        if total_steps is None:
            try:
                total_steps = len(iterable)
                if not total_steps: total_steps = None
            except:
                total_steps = None
        self.total_steps = total_steps
        self.title = title

    def __iter__(self):
        with Progress() as progress:
            task = progress.add_task(self.title, total=self.total_steps)
            for idx, item in enumerate(self.iterable):
                yield item
                progress.update(task, advance=1)


if __name__ == "__main__":
    import time
    import threading


    def test1():
        def show():
            with Progress() as progress:
                task = progress.add_task('emmm', total=1000)
                for i in probar2(range(1000), title='Download'):
                    time.sleep(.005)
                    progress.update(task, advance=1)

        results = []
        for x in 1000, 1500, 700, 0:
            t = threading.Thread(target=show, )
            results.append(t)
            t.start()
        [i.join() for i in results]


    def test2():
        with Progress() as progress:
            task1 = progress.add_task("[red]Downloading...", total=1000)
            task2 = progress.add_task("[green]Processing...", total=100)
            task3 = progress.add_task("[cyan]Cooking...", total=1000)

            while not progress.finished:
                progress.update(task1, advance=0.5)
                progress.update(task2, advance=0.3)
                progress.update(task3, advance=0.9)
                time.sleep(0.02)


    # test1()
    # test2()

    def test3():
        from time import sleep

        from rich.live import Live
        from rich.panel import Panel
        from rich.progress import Progress, SpinnerColumn, BarColumn, TextColumn
        from rich.table import Table

        job_progress = Progress(
            "{task.description}",
            SpinnerColumn(),
            BarColumn(),
            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
        )
        job1 = job_progress.add_task("[green]Cooking")
        job2 = job_progress.add_task("[magenta]Baking", total=200)
        job3 = job_progress.add_task("[cyan]Mixing", total=400)

        total = sum(task.total for task in job_progress.tasks)
        overall_progress = Progress()
        overall_task = overall_progress.add_task("All Jobs", total=int(total))

        progress_table = Table.grid()
        progress_table.add_row(
            Panel.fit(
                overall_progress, title="Overall Progress", border_style="green", padding=(2, 2)
            ),
            Panel.fit(job_progress, title="[b]Jobs", border_style="red", padding=(1, 2)),
        )

        with Live(progress_table, refresh_per_second=10):
            while not overall_progress.finished:
                sleep(0.1)
                for job in job_progress.tasks:
                    if not job.finished:
                        job_progress.advance(job.id)

                completed = sum(task.completed for task in job_progress.tasks)
                overall_progress.update(overall_task, completed=completed)


    test3()



import logging
from sparrow.log.core import Logger, SimpleLogger
from sparrow import yaml_load
from sparrow import rel_to_abs
import time
# import logging
# from logtest1 import func1
from multiprocessing import Process, context
import multiprocessing


def test_simple_log():
    # simp
    logger = SimpleLogger.get_logger(name="train-log", log_dir=rel_to_abs("./log"), print_stream=True)
    logger.debug("hello", "list", [1, 2, 3, 4, 5])
    logger2 = SimpleLogger.get_logger(name="train-log", )
    print(id(logger2) == id(logger))
    assert (id(logger2) == id(logger))


def test_simplelog_log():
    logger = SimpleLogger.get_logger("emmm")
    logger.log(logging.DEBUG, "hahahhah")


def test_logger():
    logger = Logger(name="train-log", log_dir=rel_to_abs("./log"), print_debug=True)
    logger.debug("hello", "list", [1, 2, 3, 4, 5])
    logger2 = Logger.get_logger("train-log")
    print(id(logger2) == id(logger))


def test_log():
    logger = Logger(name="train-log", log_dir=rel_to_abs("./log"), print_debug=True)
    logger.debug("hello", "list", [1, 2, 3, 4, 5])

    logger2 = Logger.get_logger("train-log")
    print(id(logger2) == id(logger))

    def get_logger(i, name="name"):
        # logger = Logger(name=name, log_dir=f"./logs/2022-01-12-{i}", print_error=True, single_mode=True, multi_process=False)
        logger = SimpleLogger(
            name=name,
            log_dir=rel_to_abs(f"./logemm/2022-01-12-{i}"),
            print_stream=True,
            multi_process=False,
        )
        return logger

    logger1 = get_logger(1, "emm")

    # def func1(n=2):
    #     with multiprocessing.Pool(2) as p:
    #         p.apply(func=f, args=(n,))

    def f(n, logger):
        for i in range(n):
            time.sleep(0.01)
            logger.debug("em 我在debug")
            # logger.info("额")
            # logger.warning(f"emmmm, {i}")
            logger.error("粗错啦")

    f(2, logger1)



import fire
from .widgets import timer
from .multiprocess import kill
from .multiprocess import start_server
from .docker import save_all_images as save_docker_images
from .docker import load_dir_images as load_docker_images
from .utils.compress import pack, unpack

func_list = [
    timer,
    save_docker_images,
    load_docker_images,
    pack,
    unpack,
    start_server,
    kill,
]
func_dict = {}
for func in func_list:
    func_dict[func.__name__] = func


def main():
    fire.Fire(func_dict)



# import copy
# import ml_collections
#
#
# referenced_dict = {"inner_float": 3.14}
# d = {
#     "referenced_dict_1": referenced_dict,
#     "referenced_dict_2": referenced_dict,
#     "list_containing_dict": [{"key": "value"}],
# }
#
# # We can initialize on a dictionary
# cfg = ml_collections.ConfigDict(d)
#
# print(cfg)



from sparrow.version_ops import VersionControl
from git.repo import Repo


pkgname = "sparrow_tool"
pkgdir = "sparrow"
# vc = VersionControl(pkgname, pkgdir, version="0.2.3")
vc = VersionControl(pkgname, pkgdir, version=None)
old_version = vc.config['version']

vc.update_version()
new_version = vc.config['version']
vc.update_readme(license='MIT')
# os.system("black ./sparrow")
# vc.upload_pypi()

repo = Repo('.')
repo.index.add(["README*.md", "workflow.py", 'setup.*', 'sparrow/version-config.yaml'])
repo.index.commit(f"[Upgrade] Bump version ({old_version} -> {new_version})")
tag = f"v{new_version}"
repo.create_tag(tag)
remote = repo.remote()
remote.push(tag)
remote.push()



import os
import chevron
from ..path import rel_to_abs, rel_path_join


class expand_template:
    """
    Example:
        expand_template(
            name='config_gen.py',
            template='./config.template.ts',
            out="../web/apps/chatroom/src/",
            substitutions={
                "xxxkey": "xxxvalue",
            },
        )
    """

    def __init__(self, name: str, out: str, substitutions: dict, template: str):
        self.name = name
        self.out = out
        self.substitutions = substitutions
        self.template = template
        self._gen_code()

    @staticmethod
    def write_targe_path(target_path: str, content: str):
        with open(target_path, 'w') as f:
            f.write(content)

    def _gen_code(self):
        with open(rel_to_abs(self.template), 'r') as f:
            result = chevron.render(f, self.substitutions)
            print(result)
            self.write_targe_path(rel_path_join(self.out, self.name), result)




from sparrow.performance import MeasureTime
import time


def test_show_time():
    tt = MeasureTime()
    tt.start()
    time.sleep(0.1)
    tt.show_interval()
    time.sleep(0.12)
    tt.show_interval()



from pathlib import Path
import inspect
import os


def rel_to_abs(rel_path: str, parents=0, return_str=True, strict=False):
    """Return absolute path relative to the called file
    args:
        parent: <int> The number of times `f_back` will be calledd.
    """
    currentframe = inspect.currentframe()
    f = currentframe.f_back
    if parents:
        for _ in range(parents):
            f = f.f_back
    current_path = Path(f.f_code.co_filename).parent
    pathlib_path = current_path / rel_path
    pathlib_path = pathlib_path.resolve(strict=strict)
    if return_str:
        return str(pathlib_path)
    else:
        return pathlib_path


def rel_path_join(*paths: str):
    return rel_to_abs(os.path.join(*paths))



from typing import Tuple, List, NamedTuple
import psutil


class ProcessInfo(NamedTuple):
    port: int
    process: psutil.Process


def get_processes(ports: List[int]) -> List[ProcessInfo]:
    processes = set()
    for process in psutil.process_iter():
        try:
            conns = process.connections(kind="inet")
        except (psutil.AccessDenied, psutil.ZombieProcess):
            continue

        for conn in conns:
            port = conn.laddr.port
            if port in ports:
                processes.add(ProcessInfo(port=port, process=process))

    return sorted(processes, key=lambda p: p.port)


def kill_ports(ports: Tuple[int], just_view: bool = False) -> int:
    processes = get_processes(ports)
    if not processes:
        print("🙃 Do not find process by the given ports")
        return False

    for pinfo in processes:
        emoji = "👁️" if just_view else "☠️🔪"
        process = pinfo.process
        if not just_view:
            process.kill()
        print(
            f"{emoji}: {process.name()} (pid {process.pid}) "
            f"on port {pinfo.port}",
        )
    return True if processes else False



import time
from ..string.color_string import rgb_string, color_const

__all__ = ['MeasureTime']


class MeasureTime:
    def __init__(self, prec=2, logger=None):
        self._cost_time = 0.
        self._start_time = time.time()
        self._logger = logger
        self._msg = None
        self._prec: int = prec

    def start(self):
        self._start_time = time.time()
        return self

    def end(self):
        self._cost_time = time.time() - self._start_time
        return self

    def show_interval(self, msg=None):
        self._msg = msg
        self._cost_time = time.time() - self._start_time
        self._start_time = time.time()
        self._show_cost()
        return self.get_cost()

    def get_cost(self):
        return f"{self._cost_time:.{int(self._prec)}E}"

    def _show_cost(self):
        cost_time = self.get_cost()
        msg = f"{self._msg}\t" if self._msg else ''
        if self._logger:
            show_string = f"{msg}cost time: {cost_time}s"
            self._logger.debug(show_string)
        else:
            rgb_cost_time = rgb_string(cost_time, color=color_const.GREEN)
            rgb_msg = rgb_string(f"{msg}", color=color_const.cyan)
            show_string = f"{rgb_msg}cost time: {rgb_cost_time}s"
            print(show_string)



if __name__ == "__main__":
    from sparrow.performance import MeasureTime
    from fast_jieba.analyse import extract_tags, textrank
    import jieba
    import time
    import fast_jieba

    ms = MeasureTime()

    import random
    tt = []
    t = "小明就读北京清华大学物理系"
    for i in range(1000):
        idx = random.randint(0, len(t) - 1)
        t += t[idx]
        t = t[idx] + t
        t += t[idx]
        t = t[idx] + t
        tt.append(t)

    # texts = [tt for _ in range(4000)]
    # print(tt[:2])
    texts = tt
    ms.start()
    [fast_jieba.tokenize(text) for text in texts]
    ms.show_interval(msg='fast jieba')

    words = fast_jieba.batch_tokenize(texts, )
    ms.show_interval('batch fast jieba')

    [list(jieba.tokenize(text)) for text in texts]
    ms.show_interval(msg="jieba")
